﻿:Class Monitor
⍝  Implement "Monitor" User Command.

    ⎕IO←1 ⋄ ⎕ML←0 ⍝ Set those here to avoid inheriting them from outside

    ∇ r←Help Cmd
      :Access Shared Public
      :Select Cmd
      :Case 'Monitor'
          r←'monitor  on|off|report -fns= -top=' ''
          r,←⊂'on            switches monitoring on'
          r,←⊂'off           switches monitoring off'
          r,←⊂'report        displays monitor report'
          r,←'' 'switches:'
          r,←⊂' -fns=fn1,fn2  only for named fns'
          r,←⊂'-top=n        only report on top n fns'
          r,←⊂'-min=n        only report lines/fns consuming n% (default 1)'
          r←⍪r
      :Case 'APLMON'
          r←'aplmon on|off -file=' ''
          r,←⊂'on            switches monitoring on'
          r,←⊂'off           switches monitoring off'
          r,←'' 'switch:'
          r,←⊂' -file=filename   (only required when turing aplmon on)'
          r←⍪r
     
      :Case 'CPUTime'
          r←⍪'cputime expression' ''
     
      :EndSelect
    ∇

    ∇ r←List ⍝ this function usually returns 1 or more namespaces (here only 1)
      :Access Shared Public
      r←,⎕NS¨'' '' ''              ⍝ create 2 commands
      r.Name←'Monitor' 'APLMON' 'CPUTime' ⍝ their names
      r.Desc←'Set/Report ⎕MONITOR' 'Turn APLMON on or off' 'Time execution of an expression' ⍝ descriptions
      r.Group←⊂'Tools'             ⍝ same group
      r[1].Parse←'1 -fns= -top= -caption= -minimum='⍝ 'monitor' takes 1 argument and swithes -Fns= and -Top=
      r[2].Parse←'1 -file='⍝ 'monitor' takes 1 argument and switch -file='
      r[3].Parse←,'1'
    ∇

    ∇ r←Run(Cmd Args);⎕TRAP;cs;fns;top;min;switch;expr;cap;file;ai
      :Access Shared Public
     
      ⎕TRAP←0 'S' ⍝ While we debug this thang
      cs←⍎(⎕XSI⍳⊂'⎕SE.UCMD')⊃⎕NSI,⊂'#'
     
      :Select Cmd
      :Case 'Monitor'
     
          fns←{(⍵∧.≠',')/1↓¨(','=⍵)⊂⍵}',',''Args.Switch'fns'
          :If 0∊⍴fns ⋄ fns←↓cs.⎕NL 3 ⋄ :EndIf
          top←⊃2⊃⎕VFI''Args.Switch'top'
          min←0.01×{⍵+1×⍵=0}⊃2⊃⎕VFI''Args.Switch'minimum'
          cap←{0≡⍵:'' ⋄ ⍵}''Args.Switch'caption'
     
          :Select 3↑switch←1⊃Args.Arguments
          :CaseList 'on ' 'off'
              r←('on'≡2↑switch)MonSet cs fns
          :Case 'rep' ⍝ report
              r←MonRep cs fns top min cap
          :Case 'run'
              expr←⍕1↓Args.Arguments
              ⎕←1 MonSet cs fns
              ⎕←'Running: ',expr
              cs⍎expr
              r←MonRep cs fns top min
          :Case 'cle' ⍝ clear
              ⎕EX'⎕SE.MonitorData'
          :Else ⋄ r←Help Cmd
          :EndSelect
     
      :Case 'APLMON'
     
          file←''Args.Switch'file'
     
          :Select 3↑switch←1⊃Args.Arguments
          :Case 'on '
              r←2 ⎕NQ'.' 'APLMON'(file,(~'.'∊file)/'.csv')
              r←'was: ',r,(0=⍴r)/'OFF'
          :Case 'off'
              r←2 ⎕NQ'.' 'APLMON' ''
              r←'was: ',r,(0=⍴r)/'OFF'
          :EndSelect
     
      :Case 'CPUTime'
     
          ai←⎕AI ⋄ {}{6::0 ⋄ cs⍎⍵}1⊃Args.Arguments ⋄ ai←⎕AI-ai
          r←'CPU:' 'Elapsed:',[1.5]ai[2 3]
     
      :EndSelect
    ∇

    ∇ r←on MonSet(space fns);f
      :For f :In fns
          (on/0,⍳1↑⍴cs.⎕NR f)cs.⎕MONITOR f
      :EndFor
      r←'Monitoring switched ',((1+on)⊃'off' 'on'),' for ',(⍕1↑⍴fns),' functions'
    ∇

    ∇ r←MonRep(space fns top min cap);mondata;max;all;i;ix;z;m
     
      :If 0≠⍴fns
          mondata←cs.⎕MONITOR¨fns
          max←⌈⌿all←↑2↓¨4⍴¨mondata
      :AndIf 0∨.≠max
          i←{⍵/⍳⍴⍵}(all÷[2]max)∨.≥min
          top←(⍴i)⌊top+(0=top)×⍴i
          i←i[top↑⍒all[i;]]
     
          r←0 7⍴⊂''
          :For ix :In i
              z←ix⊃mondata
              z←(m←(z[;3 4]÷[2]max)∨.>min)⌿z
              z←z[;2 3 4],(100×z[;3]÷max[1]),z[;,1]
              r←r⍪(fns[ix]),z,↓m⌿cs.⎕CR ix⊃fns
     
         ⍝ r←r,⊂('# REPS    CPU   Time     % Line '⍪'CI6,I7,I7,F6.1,X1,P<[>Q<]>LBI5'⎕FMT z[;2 3 4],(100×z[;3]÷max[1]),z[;,1]),' '⍪m⌿cs.⎕CR ix⊃fns
          :EndFor
          AddTab cap r
      ⍝r←⍪('Highest CPU & Time is ',⍕max)'',' '⍪¨r
      :Else
          r←'(no monitor data found)'
      :EndIf
    ∇

    ∇ {larg}MonitorDataHandler msg;sf;m;fn
     ⍝ Handle Callbacks on the Grid
     
      sf←⎕SE.MonitorData.TC.((Buttons.State=1)/SubForms)
      :Select larg
     
      :Case 'Close'
          ⎕SE.MonitorData.TC.((Buttons SubForms)←(0=Buttons.State)∘/¨Buttons SubForms) ⍝ Close active subform
      :Case 'Copy'
          :If 0≠⍴sf ⋄ (⊃sf).G.GridCopy ⍬ ⋄ :EndIf ⍝ Copy to clipboard
      :Case 'Edit'
          :If 0≠⍴sf ⋄
              m←0≠⊃∘⍴¨fn←(⊃sf).G.Values[;1]
              fn←(+\m)[7⊃msg]⊃m/fn
              :If 3=#.⎕NC fn
                  #.⎕ED fn
              :EndIf
          :EndIf
      :Case 'Finalize'
          ⎕EX'⎕SE.MonitorData'
      :EndSelect
     
    ∇

    ∇ TC←Make;form
      :If 9=⎕NC'⎕SE.MonitorData.TC'
      :AndIf 'Form'≡'⎕SE.MonitorData'⎕WG'Type'
          TC←⎕SE.MonitorData.TC
      :Else
          form←⎕SE.(MonitorData←⎕NEW'Form'(('Caption' ']monitor output')('Coord' 'Pixel')('Size'(400 800))))
          ⎕SE.Dyalog.⎕FX ⎕CR'MonitorDataHandler'
          form ⎕WS'Event'('Close' 'MonitorDataHandler' 'Finalize')
          form.(MB←⎕NEW'MenuBar'⍬)
          form.MB.(AC←⎕NEW'Menu'(⊂'Caption' '&Action'))
          form.MB.AC.(CL←⎕NEW'MenuItem'(('Caption' 'C&lose Active Tab')('Event'('Select' '⎕SE.Dyalog.MonitorDataHandler' 'Close'))))
          form.MB.AC.(CP←⎕NEW'MenuItem'(('Caption' '&Copy to Clipboard')('Event'('Select' '⎕SE.Dyalog.MonitorDataHandler' 'Copy'))))
     
          TC←⎕SE.MonitorData.(TC←⎕NEW'TabControl'⍬)
          TC.(Buttons←SubForms←⍬)
      :EndIf
    ∇
    
    ∇ SubForm←AddTab(Label Data);id;tb;form;props;newfn
     ⍝ Add a SubForm with a new Monitor Report
     
      TC←Make ⍝ ⎕SE.MonitorData.TC
      id←⍕{⊃(⍳1+⍴⍵)~⍵}2⊃⎕VFI⍕1↓¨'T'TC.⎕NL-9
      :If 0=⍴TC.Buttons ⋄ Label←Label,'1'
      :Else ⋄ Label←⊃(Label∘,¨⍕¨⍳1+⍴TC.Buttons)~TC.Buttons.Caption
      :EndIf
     
      TC.Buttons,←tb←TC.⎕NEW'TabButton'(⊂'Caption'Label)
      TC.SubForms,←form←TC.⎕NEW'SubForm'(⊂'TabObj'tb)
     
      newfn←1,2≢/Data[;1] ⍝ Where do we have new fns?
      Data[(~newfn)/⍳⍴newfn;1]←⊂''
     
      props←('Values'Data)('Size'form.Size)('Attach'('Top' 'Left' 'Bottom' 'Right'))
      props,←('TitleWidth' 0)('ColTitles'('Function' 'Count' 'CPU' 'Elapsed' '%CPU' '' 'Code'))
      props,←('FormatString'('CI8' 'F6.1' 'P<[>Q<]>BI6' ''))('CellTypes'((⍴Data)⍴4 1 1 1 2 3 4))
      props,←('GridLineFCol'((255 255 255)(127 127 127)(0 0 0)))('RowLineTypes'(1+2×1⌽newfn))('ColLineTypes'((¯1↑⍴Data)⍴2 2 2 2 2 1 2))
      props,←('CellHeights'(17+10×newfn))('Event'('CellDblClick' '⎕SE.Dyalog.MonitorDataHandler' 'Edit'))
      form.G←form.⎕NEW'Grid'props
      {form.G.SetColSize ⍵ ¯3}¨¯1+⍳¯1↑⍴Data
     ⍝ Put tip on with timestamp
    ∇

:EndClass
